@@ -15,7 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * Credits go 
  * - for the SPDIF A/52 sync part
@@ -24,9 +24,6 @@
  * for initial ALSA 0.9.x support.
  *     adding MONO/STEREO/4CHANNEL/5CHANNEL/5.1CHANNEL analogue support.
  * (c) 2001 James Courtier-Dutton <James@superbug.demon.co.uk>
- *
- * 
- * $Id: audio_alsa_out.c,v 1.168 2007/02/25 22:33:25 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -859,7 +856,11 @@ static int ao_alsa_write(ao_driver_t *this_gen, int16_t *data, uint32_t count) {
 static void ao_alsa_close(ao_driver_t *this_gen) {
   alsa_driver_t *this = (alsa_driver_t *) this_gen;
 
-  if(this->audio_fd) snd_pcm_close(this->audio_fd);
+  if(this->audio_fd) {
+    snd_pcm_nonblock(this->audio_fd, 0);
+    snd_pcm_drain(this->audio_fd);
+    snd_pcm_close(this->audio_fd);
+  }
   this->audio_fd = NULL;
   this->has_pause_resume = 0; /* This is set at open time */
 }
@@ -1349,7 +1350,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   #define A52_PASSTHRU	12
   int speakers;
 
-  this = (alsa_driver_t *) xine_xmalloc (sizeof (alsa_driver_t));
+  this = calloc(1, sizeof (alsa_driver_t));
   if (!this)
     return NULL;
 
@@ -1496,70 +1497,73 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
 			  "formats you want to play to your sound card's digital output."),
 			0, alsa_speaker_arrangement_cb, this);
 
-  xprintf(class->xine, XINE_VERBOSITY_LOG, _("audio_alsa_out : supported modes are "));
+  char *logmsg = strdup (_("audio_alsa_out : supported modes are"));
+
   if (!(snd_pcm_hw_params_test_format(this->audio_fd, params, SND_PCM_FORMAT_U8))) {
     this->capabilities |= AO_CAP_8BITS;
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("8bit "));
+    xine_strcat_realloc (&logmsg, _(" 8bit"));
   }
   /* ALSA automatically appends _LE or _BE depending on the CPU */
   if (!(snd_pcm_hw_params_test_format(this->audio_fd, params, SND_PCM_FORMAT_S16))) {
     this->capabilities |= AO_CAP_16BITS;
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("16bit "));
+    xine_strcat_realloc (&logmsg, _(" 16bit"));
   }
   if (!(snd_pcm_hw_params_test_format(this->audio_fd, params, SND_PCM_FORMAT_S24))) {
     this->capabilities |= AO_CAP_24BITS;
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("24bit "));
+    xine_strcat_realloc (&logmsg, _(" 24bit"));
   }
   if (!(snd_pcm_hw_params_test_format(this->audio_fd, params, SND_PCM_FORMAT_FLOAT))) {
     this->capabilities |= AO_CAP_FLOAT32;
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("32bit "));
+    xine_strcat_realloc (&logmsg, _(" 32bit"));
   }
   if (0 == (this->capabilities & (AO_CAP_FLOAT32 | AO_CAP_24BITS | AO_CAP_16BITS | AO_CAP_8BITS))) {
-    xprintf (this->class->xine, XINE_VERBOSITY_DEBUG,
-        "\naudio_alsa_out: no supported PCM format found\n");
+    xprintf(class->xine, XINE_VERBOSITY_LOG, "%s\n", logmsg);
+    free (logmsg);
+    xprintf (class->xine, XINE_VERBOSITY_DEBUG,
+             "audio_alsa_out: no supported PCM format found\n");
     snd_pcm_close(this->audio_fd);
     free(this);
     return NULL;
   }
   if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 1))) {
     this->capabilities |= AO_CAP_MODE_MONO;
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("mono "));
+    xine_strcat_realloc (&logmsg, _(" mono"));
   }
   if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 2))) {
     this->capabilities |= AO_CAP_MODE_STEREO;
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("stereo "));
+    xine_strcat_realloc (&logmsg, _(" stereo"));
   }
   if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 4)) &&
      ( speakers == SURROUND4 )) {
     this->capabilities |= AO_CAP_MODE_4CHANNEL;
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("4-channel "));
+    xine_strcat_realloc (&logmsg, _(" 4-channel"));
   } 
   else
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("(4-channel not enabled in xine config) "));
+    xine_strcat_realloc (&logmsg, _(" (4-channel not enabled in xine config)"));
   
   if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 6)) && 
      ( speakers == SURROUND41 )) {
     this->capabilities |= AO_CAP_MODE_4_1CHANNEL;
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("4.1-channel "));
+    xine_strcat_realloc (&logmsg, _(" 4.1-channel"));
   } 
   else
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("(4.1-channel not enabled in xine config) "));
+    xine_strcat_realloc (&logmsg, _(" (4.1-channel not enabled in xine config)"));
 
   if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 6)) && 
      ( speakers == SURROUND5 )) {
     this->capabilities |= AO_CAP_MODE_5CHANNEL;
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("5-channel "));
+    xine_strcat_realloc (&logmsg, _(" 5-channel"));
   } 
   else
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("(5-channel not enabled in xine config) "));
+    xine_strcat_realloc (&logmsg, _(" (5-channel not enabled in xine config)"));
 
   if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 6)) && 
      ( speakers >= SURROUND51 )) {
     this->capabilities |= AO_CAP_MODE_5_1CHANNEL;
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("5.1-channel "));
+    xine_strcat_realloc (&logmsg, _(" 5.1-channel"));
   } 
   else
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("(5.1-channel not enabled in xine config) "));
+    xine_strcat_realloc (&logmsg, _(" (5.1-channel not enabled in xine config)"));
 
   this->has_pause_resume = 0; /* This is checked at open time instead */
   this->is_paused = 0;
@@ -1582,10 +1586,13 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   if ( speakers == A52_PASSTHRU ) {
     this->capabilities |= AO_CAP_MODE_A52;
     this->capabilities |= AO_CAP_MODE_AC5;
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("a/52 and DTS pass-through\n"));
+    xine_strcat_realloc (&logmsg, _(" a/52 and DTS pass-through"));
   } 
   else
-    xprintf(class->xine, XINE_VERBOSITY_LOG, _("(a/52 and DTS pass-through not enabled in xine config)\n"));
+    xine_strcat_realloc (&logmsg, _(" (a/52 and DTS pass-through not enabled in xine config)"));
+
+  xprintf(class->xine, XINE_VERBOSITY_LOG, "%s\n", logmsg);
+  free (logmsg);
 
   /* printf("audio_alsa_out: capabilities 0x%X\n",this->capabilities); */
 
@@ -1681,7 +1688,7 @@ static void *init_class (xine_t *xine, void *data) {
 
   alsa_class_t        *this;
 
-  this = (alsa_class_t *) xine_xmalloc (sizeof (alsa_class_t));
+  this = calloc(1, sizeof (alsa_class_t));
   if (!this)
     return NULL;
 
