@@ -530,7 +530,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen,
 
   lprintf ("open_plugin called\n");
 
-  this = (coreaudio_driver_t *) xine_xmalloc (sizeof (coreaudio_driver_t));
+  this = calloc(1, sizeof (coreaudio_driver_t));
   if (!this)
     return NULL;
 
@@ -582,7 +582,7 @@ static void *init_class (xine_t *xine, void *data) {
 
   lprintf ("init class\n");
 
-  this = (coreaudio_class_t *) xine_xmalloc (sizeof (coreaudio_class_t));
+  this = calloc(1, sizeof (coreaudio_class_t));
   if (!this)
     return NULL;
 
