@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
@@ -33,8 +33,6 @@
  * initialized to 0 at the start of the file and maintained throughout the
  * data. This makes seeking conceptually impossible. Upshot: Random
  * seeking is not supported.
- *
- * $Id: demux_aud.c,v 1.20 2007/01/19 00:26:40 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -272,7 +270,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_aud_t    *this;
 
-  this         = xine_xmalloc (sizeof (demux_aud_t));
+  this         = calloc(1, sizeof(demux_aud_t));
   this->stream = stream;
   this->input  = input;
 
@@ -345,7 +343,7 @@ static void class_dispose (demux_class_t *this_gen) {
 void *demux_aud_init_plugin (xine_t *xine, void *data) {
   demux_aud_class_t     *this;
 
-  this = xine_xmalloc (sizeof (demux_aud_class_t));
+  this = calloc(1, sizeof(demux_aud_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
