@@ -347,7 +347,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_raw_dv_t *this;
 
-  this         = xine_xmalloc (sizeof (demux_raw_dv_t));
+  this         = calloc(1, sizeof(demux_raw_dv_t));
   this->stream = stream;
   this->input  = input;
 
@@ -436,7 +436,7 @@ static void class_dispose (demux_class_t *this_gen) {
 static void *init_plugin (xine_t *xine, void *data) {
   demux_raw_dv_class_t     *this;
 
-  this = xine_xmalloc (sizeof (demux_raw_dv_class_t));
+  this = calloc(1, sizeof(demux_raw_dv_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
