@@ -478,7 +478,7 @@ static vo_frame_t* syncfb_alloc_frame(vo_driver_t* this_gen)
   /* syncfb_driver_t *this = (syncfb_driver_t *) this_gen; */
   syncfb_frame_t  *frame;
   
-  frame = (syncfb_frame_t *) xine_xmalloc(sizeof(syncfb_frame_t));
+  frame = calloc(1, sizeof(syncfb_frame_t));
   if(!frame)
     return NULL;
   
@@ -882,7 +882,7 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
 
    display     = visual->display;
    
-   if(!(this = xine_xmalloc(sizeof (syncfb_driver_t))))
+   if(!(this = calloc(1, sizeof (syncfb_driver_t))))
      return NULL;
  
    _x_alphablend_init(&this->alphablend_extra_data, class->xine);
@@ -1105,7 +1105,7 @@ static void *init_class (xine_t *xine, void *visual_gen) {
   /*
    * from this point on, nothing should go wrong anymore
    */
-  this = (syncfb_class_t *) xine_xmalloc (sizeof (syncfb_class_t));
+  this = calloc(1, sizeof (syncfb_class_t));
 
   this->driver_class.open_plugin     = open_plugin;
   this->driver_class.get_identifier  = get_identifier;
