@@ -448,7 +448,7 @@ struct video_overlay_manager_s {
  * build a video_out_port from
  * a given video driver
  */
-xine_video_port_t *_x_vo_new_port (xine_t *xine, vo_driver_t *driver, int grabonly) XINE_PROTECTED;
+xine_video_port_t *_x_vo_new_port (xine_t *xine, vo_driver_t *driver, int grabonly) XINE_MALLOC XINE_PROTECTED;
 
 #ifdef __cplusplus
 }
